@@ -7,6 +7,11 @@
 __attribute_cold__
 void connections_free(server *srv);
 
+__attribute_cold__
+void connection_graceful_shutdown_maint (server *srv);
+
+void connection_periodic_maint (server *srv, time_t cur_ts);
+
 connection * connection_accept(server *srv, server_socket *srv_sock);
 connection * connection_accepted(server *srv, server_socket *srv_socket, sock_addr *cnt_addr, int cnt);
 
