@@ -44,30 +44,11 @@ static int data_string_insert_dup(data_unset *dst, data_unset *src) {
 		buffer_copy_buffer(ds_dst->value, ds_src->value);
 	}
 
-	src->free(src);
+	src->fn->free(src);
 
 	return 0;
 }
 
-static int data_response_insert_dup(data_unset *dst, data_unset *src) {
-	data_string *ds_dst = (data_string *)dst;
-	data_string *ds_src = (data_string *)src;
-
-	if (!buffer_is_empty(ds_dst->value)) {
-		buffer_append_string_len(ds_dst->value, CONST_STR_LEN("\r\n"));
-		buffer_append_string_buffer(ds_dst->value, ds_dst->key);
-		buffer_append_string_len(ds_dst->value, CONST_STR_LEN(": "));
-		buffer_append_string_buffer(ds_dst->value, ds_src->value);
-	} else {
-		buffer_copy_buffer(ds_dst->value, ds_src->value);
-	}
-
-	src->free(src);
-
-	return 0;
-}
-
-
 static void data_string_print(const data_unset *d, int depth) {
 	data_string *ds = (data_string *)d;
 	size_t i, len;
@@ -95,6 +76,13 @@ static void data_string_print(const data_unset *d, int depth) {
 
 
 data_string *data_string_init(void) {
+	static const struct data_methods fn = {
+		data_string_reset,
+		data_string_copy,
+		data_string_free,
+		data_string_insert_dup,
+		data_string_print,
+	};
 	data_string *ds;
 
 	ds = calloc(1, sizeof(*ds));
@@ -103,21 +91,8 @@ data_string *data_string_init(void) {
 	ds->key = buffer_init();
 	ds->value = buffer_init();
 
-	ds->copy = data_string_copy;
-	ds->free = data_string_free;
-	ds->reset = data_string_reset;
-	ds->insert_dup = data_string_insert_dup;
-	ds->print = data_string_print;
 	ds->type = TYPE_STRING;
-
-	return ds;
-}
-
-data_string *data_response_init(void) {
-	data_string *ds;
-
-	ds = data_string_init();
-	ds->insert_dup = data_response_insert_dup;
+	ds->fn = &fn;
 
 	return ds;
 }
