@@ -6,7 +6,7 @@
 #include <sys/stat.h>
 #include <string.h>
 
-int etag_is_equal(buffer *etag, const char *line, int weak_ok) {
+int etag_is_equal(const buffer *etag, const char *line, int weak_ok) {
 	enum {
 		START = 0,
 		CHECK,
@@ -143,7 +143,7 @@ int etag_is_equal(buffer *etag, const char *line, int weak_ok) {
 	return 0;
 }
 
-int etag_create(buffer *etag, struct stat *st,etag_flags_t flags) {
+int etag_create(buffer *etag, const struct stat *st, etag_flags_t flags) {
 	if (0 == flags) return 0;
 
 	buffer_clear(etag);
@@ -160,6 +160,13 @@ int etag_create(buffer *etag, struct stat *st,etag_flags_t flags) {
 	
 	if (flags & ETAG_USE_MTIME) {
 		buffer_append_int(etag, st->st_mtime);
+	      #ifdef st_mtime /* use high-precision timestamp if available */
+	      #if defined(__APPLE__) && defined(__MACH__)
+		buffer_append_int(etag, st->st_mtimespec.tv_nsec);
+	      #else
+		buffer_append_int(etag, st->st_mtim.tv_nsec);
+	      #endif
+	      #endif
 	}
 
 	return 0;
