@@ -200,11 +200,7 @@ static int gw_extension_insert(gw_exts *ext, buffer *key, gw_host *fh) {
         fe->last_used_ndx = -1;
         buffer_copy_buffer(fe->key, key);
 
-        if (ext->size == 0) {
-            ext->size = 8;
-            ext->exts = malloc(ext->size * sizeof(*(ext->exts)));
-            force_assert(ext->exts);
-        } else if (ext->used == ext->size) {
+        if (ext->used == ext->size) {
             ext->size += 8;
             ext->exts = realloc(ext->exts, ext->size * sizeof(*(ext->exts)));
             force_assert(ext->exts);
@@ -435,11 +431,7 @@ static int env_add(char_array *env, const char *key, size_t key_len, const char
         }
     }
 
-    if (env->size == 0) {
-        env->size = 16;
-        env->ptr = malloc(env->size * sizeof(*env->ptr));
-        force_assert(env->ptr);
-    } else if (env->size == env->used + 1) {
+    if (env->size <= env->used + 1) {
         env->size += 16;
         env->ptr = realloc(env->ptr, env->size * sizeof(*env->ptr));
         force_assert(env->ptr);
@@ -570,7 +562,7 @@ static int gw_spawn_connection(server *srv, gw_host *host, gw_proc *proc, int de
             env.ptr[env.used] = NULL;
         }
 
-        dfd = fdevent_open_dirname(host->args.ptr[0]);
+        dfd = fdevent_open_dirname(host->args.ptr[0], 1); /* permit symlinks */
         if (-1 == dfd) {
             log_error_write(srv, __FILE__, __LINE__, "sss",
                             "open dirname failed:", strerror(errno),
@@ -738,10 +730,7 @@ static int parse_binpath(char_array *env, buffer *b) {
         case '\t':
             /* a WS, stop here and copy the argument */
 
-            if (env->size == 0) {
-                env->size = 16;
-                env->ptr = malloc(env->size * sizeof(*env->ptr));
-            } else if (env->size == env->used) {
+            if (env->size == env->used) {
                 env->size += 16;
                 env->ptr = realloc(env->ptr, env->size * sizeof(*env->ptr));
             }
@@ -758,10 +747,7 @@ static int parse_binpath(char_array *env, buffer *b) {
         }
     }
 
-    if (env->size == 0) {
-        env->size = 16;
-        env->ptr = malloc(env->size * sizeof(*env->ptr));
-    } else if (env->size == env->used) { /*need one extra for terminating NULL*/
+    if (env->size == env->used) { /*need one extra for terminating NULL*/
         env->size += 16;
         env->ptr = realloc(env->ptr, env->size * sizeof(*env->ptr));
     }
@@ -769,10 +755,7 @@ static int parse_binpath(char_array *env, buffer *b) {
     /* the rest */
     env->ptr[env->used++] = strdup(start);
 
-    if (env->size == 0) {
-        env->size = 16;
-        env->ptr = malloc(env->size * sizeof(*env->ptr));
-    } else if (env->size == env->used) { /*need one extra for terminating NULL*/
+    if (env->size == env->used) { /*need one extra for terminating NULL*/
         env->size += 16;
         env->ptr = realloc(env->ptr, env->size * sizeof(*env->ptr));
     }
@@ -1072,8 +1055,6 @@ static gw_handler_ctx * handler_ctx_init(size_t sz) {
     gw_handler_ctx *hctx = calloc(1, 0 == sz ? sizeof(*hctx) : sz);
     force_assert(hctx);
 
-    hctx->fde_ndx = -1;
-
     /*hctx->response = chunk_buffer_acquire();*//*(allocated when needed)*/
 
     hctx->request_id = 0;
@@ -1123,7 +1104,6 @@ static void handler_ctx_clear(gw_handler_ctx *hctx) {
     if (hctx->response) buffer_clear(hctx->response);
 
     hctx->fd = -1;
-    hctx->fde_ndx = -1;
     hctx->reconnects = 0;
     hctx->request_id = 0;
     hctx->send_content_body = 1;
@@ -1393,7 +1373,7 @@ int gw_set_defaults_backend(server *srv, gw_plugin_data *p, data_unset *du, size
                 if (buffer_string_is_empty(host->host) &&
                     buffer_string_is_empty(host->bin_path)) {
                     log_error_write(srv, __FILE__, __LINE__, "sbsbsbs",
-                            "host or binpath have to be set in:",
+                            "host or bin-path have to be set in:",
                             da->key, "= (",
                             da_ext->key, " => (",
                             da_host->key, " ( ...");
@@ -1663,11 +1643,11 @@ void gw_set_transparent(server *srv, gw_handler_ctx *hctx) {
 
 static void gw_backend_close(server *srv, gw_handler_ctx *hctx) {
     if (hctx->fd >= 0) {
-        fdevent_event_del(srv->ev, &(hctx->fde_ndx), hctx->fd);
+        fdevent_fdnode_event_del(srv->ev, hctx->fdn);
         /*fdevent_unregister(srv->ev, hctx->fd);*//*(handled below)*/
         fdevent_sched_close(srv->ev, hctx->fd, 1);
+        hctx->fdn = NULL;
         hctx->fd = -1;
-        hctx->fde_ndx = -1;
     }
 
     if (hctx->host) {
@@ -1733,7 +1713,7 @@ static void gw_conditional_tcp_fin(server *srv, gw_handler_ctx *hctx) {
     con->conf.stream_request_body &= ~FDEVENT_STREAM_REQUEST_POLLIN;
     con->is_readable = 0;
     shutdown(hctx->fd, SHUT_WR);
-    fdevent_event_clr(srv->ev, &hctx->fde_ndx, hctx->fd, FDEVENT_OUT);
+    fdevent_fdnode_event_clr(srv->ev, hctx->fdn, FDEVENT_OUT);
 }
 
 static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
@@ -1779,7 +1759,7 @@ static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
 
         srv->cur_fds++;
 
-        fdevent_register(srv->ev, hctx->fd, gw_handle_fdevent, hctx);
+        hctx->fdn = fdevent_register(srv->ev,hctx->fd,gw_handle_fdevent,hctx);
 
         if (hctx->proc->is_local) {
             hctx->pid = hctx->proc->pid;
@@ -1788,7 +1768,7 @@ static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
         switch (gw_establish_connection(srv, hctx->host, hctx->proc, hctx->pid,
                                         hctx->fd, hctx->conf.debug)) {
         case 1: /* connection is in progress */
-            fdevent_event_set(srv->ev, &(hctx->fde_ndx), hctx->fd, FDEVENT_OUT);
+            fdevent_fdnode_event_set(srv->ev, hctx->fdn, FDEVENT_OUT);
             gw_set_state(srv, hctx, GW_STATE_CONNECT_DELAYED);
             return HANDLER_WAIT_FOR_EVENT;
         case -1:/* connection error */
@@ -1820,8 +1800,7 @@ static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
             handler_t rc = hctx->create_env(srv, hctx);
             if (HANDLER_GO_ON != rc) {
                 if (HANDLER_FINISHED != rc && HANDLER_ERROR != rc)
-                    fdevent_event_clr(srv->ev, &(hctx->fde_ndx), hctx->fd,
-                                      FDEVENT_OUT);
+                    fdevent_fdnode_event_clr(srv->ev, hctx->fdn, FDEVENT_OUT);
                 return rc;
             }
         }
@@ -1836,8 +1815,7 @@ static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
             }
         }
 
-        fdevent_event_add(srv->ev, &hctx->fde_ndx, hctx->fd,
-                          FDEVENT_IN | FDEVENT_RDHUP);
+        fdevent_fdnode_event_add(srv->ev, hctx->fdn, FDEVENT_IN|FDEVENT_RDHUP);
         gw_set_state(srv, hctx, GW_STATE_WRITE);
         /* fall through */
     case GW_STATE_WRITE:
@@ -1853,8 +1831,6 @@ static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
             ret = srv->network_backend_write(srv, hctx->fd, hctx->wb,
                                              MAX_WRITE_LIMIT);
 
-            chunkqueue_remove_finished_chunks(hctx->wb);
-
             if (ret < 0) {
                 switch(errno) {
                 case EPIPE:
@@ -1879,7 +1855,7 @@ static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
         }
 
         if (hctx->wb->bytes_out == hctx->wb_reqlen) {
-            fdevent_event_clr(srv->ev, &(hctx->fde_ndx), hctx->fd, FDEVENT_OUT);
+            fdevent_fdnode_event_clr(srv->ev, hctx->fdn, FDEVENT_OUT);
             gw_set_state(srv, hctx, GW_STATE_READ);
         } else {
             off_t wblen = hctx->wb->bytes_in - hctx->wb->bytes_out;
@@ -1895,9 +1871,9 @@ static handler_t gw_write_request(server *srv, gw_handler_ctx *hctx) {
                 }
             }
             if (0 == wblen) {
-                fdevent_event_clr(srv->ev,&hctx->fde_ndx,hctx->fd,FDEVENT_OUT);
+                fdevent_fdnode_event_clr(srv->ev, hctx->fdn, FDEVENT_OUT);
             } else {
-                fdevent_event_add(srv->ev,&hctx->fde_ndx,hctx->fd,FDEVENT_OUT);
+                fdevent_fdnode_event_add(srv->ev, hctx->fdn, FDEVENT_OUT);
             }
         }
 
@@ -1955,14 +1931,14 @@ handler_t gw_handle_subrequest(server *srv, connection *con, void *p_d) {
     if ((con->conf.stream_response_body & FDEVENT_STREAM_RESPONSE_BUFMIN)
         && con->file_started) {
         if (chunkqueue_length(con->write_queue) > 65536 - 4096) {
-            fdevent_event_clr(srv->ev, &(hctx->fde_ndx), hctx->fd, FDEVENT_IN);
+            fdevent_fdnode_event_clr(srv->ev, hctx->fdn, FDEVENT_IN);
         }
-        else if (!(fdevent_event_get_interest(srv->ev, hctx->fd) & FDEVENT_IN)){
+        else if (!(fdevent_fdnode_interest(hctx->fdn) & FDEVENT_IN)) {
             /* optimistic read from backend */
             handler_t rc;
             rc = gw_recv_response(srv, hctx);        /*(might invalidate hctx)*/
             if (rc != HANDLER_GO_ON) return rc;      /*(unless HANDLER_GO_ON)*/
-            fdevent_event_add(srv->ev, &(hctx->fde_ndx), hctx->fd, FDEVENT_IN);
+            fdevent_fdnode_event_add(srv->ev, hctx->fdn, FDEVENT_IN);
         }
     }
 
@@ -2005,7 +1981,7 @@ handler_t gw_handle_subrequest(server *srv, connection *con, void *p_d) {
                 }
                 else
                     chunkqueue_append_chunkqueue(hctx->wb, req_cq);
-                if (fdevent_event_get_interest(srv->ev,hctx->fd) & FDEVENT_OUT){
+                if (fdevent_fdnode_interest(hctx->fdn) & FDEVENT_OUT) {
                     return (r == HANDLER_GO_ON) ? HANDLER_WAIT_FOR_EVENT : r;
                 }
             }
@@ -2050,8 +2026,8 @@ static handler_t gw_recv_response(server *srv, gw_handler_ctx *hctx) {
       ? chunk_buffer_acquire()
       : hctx->response;
 
-    handler_t rc = http_response_read(srv, hctx->remote_conn, &hctx->opts,
-                                      b, hctx->fd, &hctx->fde_ndx);
+    handler_t rc =
+      http_response_read(srv, hctx->remote_conn, &hctx->opts, b, hctx->fdn);
 
     if (b != hctx->response) chunk_buffer_release(b);
 
