@@ -560,7 +560,7 @@ static handler_t mod_auth_check_basic(server *srv, connection *con, void *p_d, c
 		return mod_auth_send_401_unauthorized_basic(srv, con, require->realm);
 	}
 
-	if (0 != strncasecmp(b->ptr, "Basic ", sizeof("Basic ")-1)) {
+	if (!buffer_eq_icase_ssn(b->ptr, CONST_STR_LEN("Basic "))) {
 		return mod_auth_send_401_unauthorized_basic(srv, con, require->realm);
 	}
       #ifdef __COVERITY__
@@ -601,6 +601,7 @@ static handler_t mod_auth_check_basic(server *srv, connection *con, void *p_d, c
 	case HANDLER_ERROR:
 	default:
 		log_error_write(srv, __FILE__, __LINE__, "sbsBsB", "password doesn't match for", con->uri.path, "username:", username, ", IP:", con->dst_addr_buf);
+		con->keep_alive = 0; /*(disable keep-alive if bad password)*/
 		rc = HANDLER_UNSET;
 		break;
 	}
@@ -637,7 +638,7 @@ static void mod_auth_digest_mutate_sha256(http_auth_info_t *ai, const char *m, c
     SHA256_Update(&ctx, (unsigned char *)uri, strlen(uri));
   #if 0
     /* qop=auth-int not supported, already checked in caller */
-    if (qop && strcasecmp(qop, "auth-int") == 0) {
+    if (qop && buffer_eq_icase_ss(qop, strlen(qop), CONST_STR_LEN("auth-int"))){
         SHA256_Update(&ctx, CONST_STR_LEN(":"));
         SHA256_Update(&ctx, (unsigned char *) [body checksum], ai->dlen*2);
     }
@@ -705,7 +706,7 @@ static void mod_auth_digest_mutate_sha512_256(http_auth_info_t *ai, const char *
     SHA512_256_Update(&ctx, (unsigned char *)uri, strlen(uri));
   #if 0
     /* qop=auth-int not supported, already checked in caller */
-    if (qop && strcasecmp(qop, "auth-int") == 0) {
+    if (qop && buffer_eq_icase_ss(qop, strlen(qop), CONST_STR_LEN("auth-int"))){
         SHA512_256_Update(&ctx, CONST_STR_LEN(":"));
         SHA512_256_Update(&ctx, (unsigned char *)[body checksum], ai->dlen*2);
     }
@@ -777,7 +778,7 @@ static void mod_auth_digest_mutate_md5(http_auth_info_t *ai, const char *m, cons
     li_MD5_Update(&ctx, (unsigned char *)uri, strlen(uri));
   #if 0
     /* qop=auth-int not supported, already checked in caller */
-    if (qop && strcasecmp(qop, "auth-int") == 0) {
+    if (qop && buffer_eq_icase_ss(qop, strlen(qop), CONST_STR_LEN("auth-int"))){
         li_MD5_Update(&ctx, CONST_STR_LEN(":"));
         li_MD5_Update(&ctx, (unsigned char *) [body checksum], ai->dlen*2);
     }
@@ -868,9 +869,9 @@ static void mod_auth_digest_www_authenticate(buffer *b, time_t cur_ts, const str
         }
         buffer_append_string_len(b, CONST_STR_LEN("Digest realm=\""));
         buffer_append_string_buffer(b, require->realm);
-        buffer_append_string_len(b, CONST_STR_LEN("\", charset=\"UTF-8\", algorithm=\""));
+        buffer_append_string_len(b, CONST_STR_LEN("\", charset=\"UTF-8\", algorithm="));
         buffer_append_string_len(b, algoname[i], algolen[i]);
-        buffer_append_string_len(b, CONST_STR_LEN("\", nonce=\""));
+        buffer_append_string_len(b, CONST_STR_LEN(", nonce=\""));
         buffer_append_uint_hex(b, (uintmax_t)cur_ts);
         buffer_append_string_len(b, CONST_STR_LEN(":"));
         (append_nonce[i])(b, cur_ts, rnd);
@@ -881,6 +882,33 @@ static void mod_auth_digest_www_authenticate(buffer *b, time_t cur_ts, const str
     }
 }
 
+static void mod_auth_digest_authentication_info(buffer *b, time_t cur_ts, int dalgo) {
+    const int rnd = li_rand_pseudo();
+    void(*append_nonce)(buffer *, time_t, int);
+    switch (dalgo) {
+     #ifdef USE_OPENSSL_CRYPTO
+      #ifdef SHA512_256_DIGEST_LENGTH
+      case HTTP_AUTH_DIGEST_SHA512_256:
+        append_nonce = mod_auth_digest_nonce_sha512_256;
+        break;
+      #endif
+      case HTTP_AUTH_DIGEST_SHA256:
+        append_nonce = mod_auth_digest_nonce_sha256;
+        break;
+     #endif
+      /*case HTTP_AUTH_DIGEST_MD5:*/
+      default:
+        append_nonce = mod_auth_digest_nonce_md5;
+        break;
+    }
+    buffer_clear(b);
+    buffer_append_string_len(b, CONST_STR_LEN("nextnonce=\""));
+    buffer_append_uint_hex(b, (uintmax_t)cur_ts);
+    buffer_append_string_len(b, CONST_STR_LEN(":"));
+    (append_nonce)(b, cur_ts, rnd);
+    buffer_append_string_len(b, CONST_STR_LEN("\""));
+}
+
 typedef struct {
 	const char *key;
 	int key_len;
@@ -951,7 +979,7 @@ static handler_t mod_auth_check_digest(server *srv, connection *con, void *p_d,
 		return mod_auth_send_401_unauthorized_digest(srv, con, require, 0);
 	}
 
-	if (0 != strncasecmp(vb->ptr, "Digest ", sizeof("Digest ")-1)) {
+	if (!buffer_eq_icase_ssn(vb->ptr, CONST_STR_LEN("Digest "))) {
 		return mod_auth_send_401_unauthorized_digest(srv, con, require, 0);
 	} else {
 		size_t n = buffer_string_length(vb);
@@ -1055,7 +1083,7 @@ static handler_t mod_auth_check_digest(server *srv, connection *con, void *p_d,
 		}
 	}
 
-	if (qop && strcasecmp(qop, "auth-int") == 0) {
+	if (qop && buffer_eq_icase_ss(qop, strlen(qop), CONST_STR_LEN("auth-int"))){
 		log_error_write(srv, __FILE__, __LINE__, "s",
 				"digest: qop=auth-int not supported");
 
@@ -1076,9 +1104,7 @@ static handler_t mod_auth_check_digest(server *srv, connection *con, void *p_d,
 	 * uri sent in client request. */
 	{
 		const size_t ulen = strlen(uri);
-		const size_t rlen = buffer_string_length(con->request.orig_uri);
-		if (!buffer_is_equal_string(con->request.orig_uri, uri, ulen)
-		    && !(rlen < ulen && 0 == memcmp(con->request.orig_uri->ptr, uri, rlen) && uri[rlen] == '?')) {
+		if (!buffer_is_equal_string(con->request.orig_uri, uri, ulen)) {
 			log_error_write(srv, __FILE__, __LINE__, "sbsssB",
 					"digest: auth failed: uri mismatch (", con->request.orig_uri, "!=", uri, "), IP:", con->dst_addr_buf);
 			buffer_free(b);
@@ -1097,16 +1123,18 @@ static handler_t mod_auth_check_digest(server *srv, connection *con, void *p_d,
 		return HANDLER_FINISHED;
 	case HANDLER_ERROR:
 	default:
+		con->keep_alive = 0; /*(disable keep-alive if unknown user)*/
 		buffer_free(b);
 		return mod_auth_send_401_unauthorized_digest(srv, con, require, 0);
 	}
 
 	mod_auth_digest_mutate(&ai,m,uri,nonce,cnonce,nc,qop);
 
-	if (0 != memcmp(rdigest, ai.digest, ai.dlen)) {
+	if (!http_auth_const_time_memeq(rdigest, ai.digest, ai.dlen)) {
 		/* digest not ok */
 		log_error_write(srv, __FILE__, __LINE__, "sssB",
 				"digest: auth failed for ", username, ": wrong password, IP:", con->dst_addr_buf);
+		con->keep_alive = 0; /*(disable keep-alive if bad password)*/
 
 		buffer_free(b);
 		return mod_auth_send_401_unauthorized_digest(srv, con, require, 0);
@@ -1138,7 +1166,12 @@ static handler_t mod_auth_check_digest(server *srv, connection *con, void *p_d,
 			/* nonce is stale; have client regenerate digest */
 			buffer_free(b);
 			return mod_auth_send_401_unauthorized_digest(srv, con, require, ai.dalgo);
-		} /*(future: might send nextnonce when expiration is imminent)*/
+		}
+		else if (srv->cur_ts - ts > 540) { /*(9 mins)*/
+			/*(send nextnonce when expiration is approaching)*/
+			mod_auth_digest_authentication_info(srv->tmp_buf, srv->cur_ts, ai.dalgo);
+			http_header_response_set(con, HTTP_HEADER_OTHER, CONST_STR_LEN("Authentication-Info"), CONST_BUF_LEN(srv->tmp_buf));
+		}
 	}
 
 	http_auth_setenv(con, ai.username, ai.ulen, CONST_STR_LEN("Digest"));
