@@ -41,6 +41,7 @@
 typedef struct {
     buffer *auth_gssapi_keytab;
     buffer *auth_gssapi_principal;
+    unsigned short int auth_gssapi_store_creds;
 } plugin_config;
 
 typedef struct {
@@ -101,6 +102,7 @@ SETDEFAULTS_FUNC(mod_authn_gssapi_set_defaults) {
     config_values_t cv[] = {
         { "auth.backend.gssapi.keytab",     NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },
         { "auth.backend.gssapi.principal",  NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },
+        { "auth.backend.gssapi.store-creds",NULL, T_CONFIG_BOOLEAN,T_CONFIG_SCOPE_CONNECTION },
         { NULL,                             NULL, T_CONFIG_UNSET,  T_CONFIG_SCOPE_UNSET }
     };
 
@@ -117,6 +119,9 @@ SETDEFAULTS_FUNC(mod_authn_gssapi_set_defaults) {
 
         cv[0].destination = s->auth_gssapi_keytab;
         cv[1].destination = s->auth_gssapi_principal;
+        cv[2].destination = &s->auth_gssapi_store_creds;
+        /* default enabled for backwards compatibility; disable in future */
+        s->auth_gssapi_store_creds = 1;
 
         p->config_storage[i] = s;
 
@@ -137,6 +142,7 @@ static int mod_authn_gssapi_patch_connection(server *srv, connection *con, plugi
 
     PATCH(auth_gssapi_keytab);
     PATCH(auth_gssapi_principal);
+    PATCH(auth_gssapi_store_creds);
 
     /* skip the first, the global context */
     for (i = 1; i < srv->config_context->used; i++) {
@@ -154,6 +160,8 @@ static int mod_authn_gssapi_patch_connection(server *srv, connection *con, plugi
                 PATCH(auth_gssapi_keytab);
             } else if (buffer_is_equal_string(du->key, CONST_STR_LEN("auth.backend.gssapi.principal"))) {
                 PATCH(auth_gssapi_principal);
+            } else if (buffer_is_equal_string(du->key, CONST_STR_LEN("auth.backend.gssapi.store-creds"))) {
+                PATCH(auth_gssapi_store_creds);
             }
         }
     }
@@ -269,6 +277,13 @@ static int mod_authn_gssapi_create_krb5_ccache(server *srv, connection *con, plu
  * HTTP auth Negotiate
  */
 
+static handler_t mod_authn_gssapi_send_500_server_error (connection *con)
+{
+    con->http_status = 500;
+    con->mode = DIRECT;
+    return HANDLER_FINISHED;
+}
+
 static handler_t mod_authn_gssapi_send_401_unauthorized_negotiate (connection *con)
 {
     con->http_status = 401;
@@ -334,7 +349,7 @@ static handler_t mod_authn_gssapi_check_spnego(server *srv, connection *con, plu
     gss_name_t client_name    = GSS_C_NO_NAME;
 
     buffer *sprinc;
-    int ret = 0;
+    handler_t rc = HANDLER_UNSET;
 
     buffer *t_in = buffer_init();
     if (!buffer_append_base64_decode(t_in, realm_str, strlen(realm_str), BASE64_STANDARD)) {
@@ -421,19 +436,24 @@ static handler_t mod_authn_gssapi_check_spnego(server *srv, connection *con, plu
         goto end;
     }
 
-    if (!(acc_flags & GSS_C_DELEG_FLAG)) {
-        log_error_write(srv, __FILE__, __LINE__, "ss", "Unable to delegate credentials for user:", token_out.value);
-        goto end;
-    }
-
     /* check the allow-rules */
     if (!http_auth_match_rules(require, token_out.value, NULL, NULL)) {
         goto end;
     }
 
-    ret = mod_authn_gssapi_store_gss_creds(srv, con, p, token_out.value, client_cred);
-    if (ret)
-        http_auth_setenv(con, token_out.value, token_out.length, CONST_STR_LEN("GSSAPI"));
+    if (p->conf.auth_gssapi_store_creds) {
+        if (!(acc_flags & GSS_C_DELEG_FLAG)) {
+            log_error_write(srv, __FILE__, __LINE__, "ss", "Unable to delegate credentials for user:", token_out.value);
+            goto end;
+        }
+        else if (!mod_authn_gssapi_store_gss_creds(srv, con, p, token_out.value, client_cred)) {
+            rc = mod_authn_gssapi_send_500_server_error(con);
+            goto end;
+        }
+    }
+
+    http_auth_setenv(con, token_out.value, token_out.length, CONST_STR_LEN("GSSAPI"));
+    rc = HANDLER_GO_ON; /* success */
 
     end:
         buffer_free(t_in);
@@ -459,7 +479,7 @@ static handler_t mod_authn_gssapi_check_spnego(server *srv, connection *con, plu
         if (token_out.length)
             gss_release_buffer(&st_minor, &token_out);
 
-        return ret ? HANDLER_GO_ON : mod_authn_gssapi_send_401_unauthorized_negotiate(con);
+        return rc != HANDLER_UNSET ? rc : mod_authn_gssapi_send_401_unauthorized_negotiate(con);
 }
 
 static handler_t mod_authn_gssapi_check (server *srv, connection *con, void *p_d, const struct http_auth_require_t *require, const struct http_auth_backend_t *backend)
@@ -471,7 +491,7 @@ static handler_t mod_authn_gssapi_check (server *srv, connection *con, void *p_d
         return mod_authn_gssapi_send_401_unauthorized_negotiate(con);
     }
 
-    if (0 != strncasecmp(vb->ptr, "Negotiate ", sizeof("Negotiate ")-1)) {
+    if (!buffer_eq_icase_ssn(vb->ptr, CONST_STR_LEN("Negotiate "))) {
         return mod_authn_gssapi_send_400_bad_request(srv, con);
     }
 
@@ -719,6 +739,8 @@ static handler_t mod_authn_gssapi_basic(server *srv, connection *con, void *p_d,
         goto end;
     }
 
+    if (!p->conf.auth_gssapi_store_creds) goto end;
+
     ret = krb5_cc_resolve(kcontext, "MEMORY:", &ret_ccache);
     if (ret) {
         mod_authn_gssapi_log_krb5_error(srv, __FILE__, __LINE__, "krb5_cc_resolve", NULL, kcontext, ret);
