@@ -15,7 +15,6 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include "sys-strings.h"
 
 #include <fcntl.h>
 #include <unistd.h>
@@ -564,7 +563,6 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, in
 
 			munmap(start, sce->st.st_size);
 			close(ofd);
-			close(ifd);
 
 			/* Remove the incomplete cache file, so that later hits aren't served from it */
 			if (-1 == unlink(p->ofn->ptr)) {
@@ -579,7 +577,6 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, in
 		log_error_write(srv, __FILE__, __LINE__, "sbss", "reading", fn, "failed", strerror(errno));
 
 		close(ofd);
-		close(ifd);
 		free(start);
 
 		/* Remove the incomplete cache file, so that later hits aren't served from it */
@@ -628,8 +625,6 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, in
 #endif
 		free(start);
 
-	close(ifd);
-
 	if (0 != close(ofd) || ret != 0) {
 		if (0 == ret) {
 			log_error_write(srv, __FILE__, __LINE__, "sbss", "writing cachefile", p->ofn, "failed:", strerror(errno));
@@ -690,7 +685,6 @@ static int deflate_file_to_buffer(server *srv, connection *con, plugin_data *p,
 				fn, ifd);
 
 			munmap(start, sce->st.st_size);
-			close(ifd);
 			return -1;
 		}
 	} else
@@ -698,7 +692,6 @@ static int deflate_file_to_buffer(server *srv, connection *con, plugin_data *p,
 	if (NULL == (start = malloc(sce->st.st_size)) || sce->st.st_size != read(ifd, start, sce->st.st_size)) {
 		log_error_write(srv, __FILE__, __LINE__, "sbss", "reading", fn, "failed", strerror(errno));
 
-		close(ifd);
 		free(start);
 		return -1;
 	}
@@ -732,8 +725,6 @@ static int deflate_file_to_buffer(server *srv, connection *con, plugin_data *p,
 #endif
 		free(start);
 
-	close(ifd);
-
 	if (ret != 0) return -1;
 
 	mod_compress_note_ratio(srv, con, sce->st.st_size,
@@ -798,7 +789,7 @@ static int mod_compress_contains_encoding(const char *headervalue, const char *e
 		while (*m == ',' || *m == ' ' || *m == '\t') {
 			++m;
 		}
-		if (0 == strncasecmp(m, encoding, len)) {
+		if (buffer_eq_icase_ssn(m, encoding, len)) {
 			/*(not a full HTTP field parse: not parsing for q-values and not handling q=0)*/
 			m += len;
 			if (*m == '\0' || *m == ',' || *m == ';' || *m == ' ' || *m == '\t')
