@@ -226,7 +226,7 @@ SETDEFAULTS_FUNC(mod_expire_set_defaults) {
 
 	if (!p) return HANDLER_ERROR;
 
-	p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
+	p->config_storage = calloc(srv->config_context->used, sizeof(plugin_config *));
 
 	for (i = 0; i < srv->config_context->used; i++) {
 		data_config const* config = (data_config const*)srv->config_context->data[i];
@@ -361,9 +361,6 @@ CONNECTION_FUNC(mod_expire_handler) {
 			time_t ts, expires;
 			stat_cache_entry *sce = NULL;
 
-			/* if stat fails => sce == NULL, ignore return value */
-			(void) stat_cache_get_entry(srv, con, con->physical.path, &sce);
-
 			switch(mod_expire_get_offset(srv, p, vb, &ts)) {
 			case 0:
 				/* access */
@@ -372,6 +369,9 @@ CONNECTION_FUNC(mod_expire_handler) {
 			case 1:
 				/* modification */
 
+				/* if stat fails => sce == NULL, ignore return value */
+				(void) stat_cache_get_entry(srv, con, con->physical.path, &sce);
+
 				/* can't set modification based expire header if
 				 * mtime is not available
 				 */
