@@ -2,8 +2,8 @@
 #define _MOD_MAGNET_CACHE_H_
 #include "first.h"
 
+#include "base_decls.h"
 #include "buffer.h"
-#include "base.h"
 
 #include <lua.h>
 
