@@ -304,7 +304,7 @@ SETDEFAULTS_FUNC(mod_rrd_set_defaults) {
 	if (!p) return HANDLER_ERROR;
 
 	force_assert(srv->config_context->used > 0);
-	p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
+	p->config_storage = calloc(srv->config_context->used, sizeof(plugin_config *));
 
 	for (i = 0; i < srv->config_context->used; i++) {
 		data_config const* config = (data_config const*)srv->config_context->data[i];
