@@ -89,7 +89,7 @@ SETDEFAULTS_FUNC(mod_skeleton_set_defaults) {
 
 	if (!p) return HANDLER_ERROR;
 
-	p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
+	p->config_storage = calloc(srv->config_context->used, sizeof(plugin_config *));
 	force_assert(p->config_storage);
 
 	for (i = 0; i < srv->config_context->used; i++) {
