@@ -32,7 +32,7 @@ SETDEFAULTS_FUNC(mod_sockproxy_set_defaults) {
 		{ NULL,                        NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 
-	p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
+	p->config_storage = calloc(srv->config_context->used, sizeof(plugin_config *));
 	force_assert(p->config_storage);
 
 	for (i = 0; i < srv->config_context->used; i++) {
