@@ -17,7 +17,6 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include "sys-strings.h"
 #include <sys/wait.h>
 
 #include <ctype.h>
@@ -574,7 +573,7 @@ static int process_ssi_stmt(server *srv, connection *con, handler_ctx *p, const
 			if (!con->conf.force_lowercase_filenames
 			    ? buffer_is_equal_right_len(con->physical.path, con->physical.rel_path, remain)
 			    :(buffer_string_length(con->physical.path) >= remain
-			      && 0 == strncasecmp(con->physical.path->ptr+buffer_string_length(con->physical.path)-remain, con->physical.rel_path->ptr+i, remain))) {
+			      && buffer_eq_icase_ssn(con->physical.path->ptr+buffer_string_length(con->physical.path)-remain, con->physical.rel_path->ptr+i, remain))) {
 				buffer_copy_string_len(p->stat_fn, con->physical.path->ptr, buffer_string_length(con->physical.path)-remain);
 				buffer_append_string_len(p->stat_fn, srv->tmp_buf->ptr+i, buffer_string_length(srv->tmp_buf)-i);
 			} else {
@@ -591,7 +590,7 @@ static int process_ssi_stmt(server *srv, connection *con, handler_ctx *p, const
 		}
 
 		int fd = stat_cache_open_rdonly_fstat(p->stat_fn, &stb, con->conf.follow_symlink);
-		if (fd > 0) {
+		if (fd >= 0) {
 			time_t t = stb.st_mtime;
 
 			switch (ssicmd) {
@@ -677,7 +676,7 @@ static int process_ssi_stmt(server *srv, connection *con, handler_ctx *p, const
 				break;
 			}
 
-			if (fd > 0) close(fd);
+			if (fd >= 0) close(fd);
 		} else {
 			log_error_write(srv, __FILE__, __LINE__, "sbs",
 					"ssi: stating failed ",
