@@ -68,7 +68,7 @@ static void mod_vhostdb_dbconf_add_scheme (server *srv, buffer *host)
             if (!buffer_string_is_empty(srv->tmp_buf))
                 buffer_append_string_len(srv->tmp_buf, CONST_STR_LEN(","));
             for (j = 0; j < sizeof(schemes)/sizeof(char *); ++j) {
-                if (0 == strncasecmp(b, schemes[j], strlen(schemes[j]))) {
+                if (buffer_eq_icase_ssn(b, schemes[j], strlen(schemes[j]))) {
                     break;
                 }
             }
@@ -256,6 +256,9 @@ static LDAP * mod_authn_ldap_host_init(server *srv, vhostdb_config *s) {
         return NULL;
     }
 
+    /* restart ldap functions if interrupted by a signal, e.g. SIGCHLD */
+    ldap_set_option(ld, LDAP_OPT_RESTART, LDAP_OPT_ON);
+
     if (s->starttls) {
         /* if no CA file is given, it is ok, as we will use encryption
          * if the server requires a CAfile it will tell us */
@@ -484,7 +487,7 @@ SETDEFAULTS_FUNC(mod_vhostdb_set_defaults) {
         { NULL,            NULL, T_CONFIG_UNSET,  T_CONFIG_SCOPE_UNSET }
     };
 
-    p->config_storage = calloc(1, srv->config_context->used * sizeof(plugin_config *));
+    p->config_storage = calloc(srv->config_context->used, sizeof(plugin_config *));
 
     for (size_t i = 0; i < srv->config_context->used; ++i) {
         data_config const *config = (data_config const*)srv->config_context->data[i];
