@@ -5,7 +5,7 @@
 #include "base_decls.h"
 #include "buffer.h"
 
-int http_request_parse(server *srv, connection *con);
+int http_request_parse(server *srv, connection *con, buffer *hdrs);
 int http_request_host_normalize(buffer *b, int scheme_port);
 int http_request_host_policy(connection *con, buffer *b, const buffer *scheme);
 
