@@ -42,10 +42,10 @@ typedef int (*http_cgi_header_append_cb)(void *vdata, const char *k, size_t klen
 int http_cgi_headers(server *srv, connection *con, http_cgi_opts *opts, http_cgi_header_append_cb cb, void *vdata);
 
 handler_t http_response_parse_headers(server *srv, connection *con, http_response_opts *opts, buffer *hdrs);
-handler_t http_response_read(server *srv, connection *con, http_response_opts *opts, buffer *b, int fd, int *fde_ndx);
+handler_t http_response_read(server *srv, connection *con, http_response_opts *opts, buffer *b, fdnode *fdn);
 handler_t http_response_prepare(server *srv, connection *con);
 int http_response_buffer_append_authority(server *srv, connection *con, buffer *b);
-int http_response_redirect_to_directory(server *srv, connection *con);
+int http_response_redirect_to_directory(server *srv, connection *con, int status);
 int http_response_handle_cachable(server *srv, connection *con, buffer * mtime);
 void http_response_body_clear(connection *con, int preserve_length);
 void http_response_send_file (server *srv, connection *con, buffer *path);
