@@ -544,10 +544,9 @@ int sock_addr_from_str_hints(server *srv, sock_addr *saddr, socklen_t *len, cons
             }
             memcpy(saddr->un.sun_path, str, hostlen);
           #if defined(SUN_LEN)
-            *len = SUN_LEN(&saddr->un);
+            *len = SUN_LEN(&saddr->un)+1;
           #else
-            /* stevens says: */
-            *len = hostlen + sizeof(saddr->un.sun_family);
+            *len = offsetof(struct sockaddr_un, sun_path) + hostlen;
           #endif
         }
         return 1;
@@ -595,11 +594,12 @@ int sock_addr_from_str_numeric(server *srv, sock_addr *saddr, const char *str)
         log_error_write(srv, __FILE__, __LINE__, "SSSs(S)",
                         "could not parse ip address ", str, " because ",
                         gai_strerror(result), strerror(errno));
+        return result;
     } else if (addrlist == NULL) {
         log_error_write(srv, __FILE__, __LINE__, "SSS",
                         "Problem in parsing ip address ", str,
                         ": succeeded, but no information returned");
-        result = -1;
+        return -1;
     } else switch (addrlist->ai_family) {
     case AF_INET:
         memcpy(&saddr->ipv4, addrlist->ai_addr, sizeof(saddr->ipv4));
